var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
import { ChainId, Networks } from "../common";
import { SwapType } from "../common/swaptype";
import { contractAddressFor, executePopulatedTransaction, rejectPromise, } from "../common/utils";
import { Tokens, } from "../tokens";
import { SwapPools } from "../swappools";
import { SynapseEntities } from "../entities";
import { newProviderForNetwork } from "../rpcproviders";
import { Zero } from "@ethersproject/constants";
import { formatUnits } from "@ethersproject/units";
import { BigNumber } from "@ethersproject/bignumber";
import { UnsupportedSwapReason } from "./errors";
import { ERC20, MAX_APPROVAL_AMOUNT } from "./erc20";
import { BridgeUtils } from "./bridgeutils";
import { GasUtils } from "./gasutils";
import { BaseToken, WrappedToken } from "../token";
export var Bridge;
(function (Bridge) {
    class SynapseBridge {
        constructor(args) {
            this.bridgeConfigInstance = SynapseEntities.bridgeConfig();
            this.zapBridgeInstance = SynapseEntities.l1BridgeZap({
                chainId: ChainId.ETH,
                signerOrProvider: newProviderForNetwork(ChainId.ETH),
            });
            let { network, provider } = args;
            this.network = network instanceof Networks.Network ? network : Networks.fromChainId(network);
            this.chainId = this.network.chainId;
            this.provider = provider !== null && provider !== void 0 ? provider : newProviderForNetwork(this.chainId);
            this.requiredConfirmations = getRequiredConfirmationsForBridge(this.network);
            this.isL2Zap = this.network.zapIsL2BridgeZap;
            let factoryParams = { chainId: this.chainId, signerOrProvider: this.provider };
            this.bridgeInstance = SynapseEntities.synapseBridge(factoryParams);
            this.bridgeAddress = contractAddressFor(this.chainId, "bridge");
            this.networkZapBridgeInstance = SynapseEntities.zapBridge({ chainId: this.chainId, signerOrProvider: this.provider });
            this.zapBridgeAddress = this.networkZapBridgeInstance.address;
        }
        bridgeVersion() {
            return this.bridgeInstance.bridgeVersion();
        }
        WETH_ADDRESS() {
            return this.bridgeInstance.WETH_ADDRESS();
        }
        swapSupported(args) {
            let { tokenFrom, tokenTo, chainIdTo } = args;
            if (!this.network.supportsToken(tokenFrom)) {
                return [false, UnsupportedSwapReason.TokenNotSupported_From];
            }
            if (!Networks.networkSupportsToken(chainIdTo, tokenTo)) {
                return [false, UnsupportedSwapReason.TokenNotSupported_To];
            }
            if (tokenFrom.swapType !== tokenTo.swapType) {
                return [false, UnsupportedSwapReason.NonmatchingSwapTypes];
            }
            let isEthFromBoba = (this.chainId === ChainId.BOBA) && (tokenFrom.swapType === SwapType.ETH), isEthToBoba = (chainIdTo === ChainId.BOBA) && (tokenTo.swapType === SwapType.ETH);
            if (isEthFromBoba || isEthToBoba) {
                return [false, UnsupportedSwapReason.ETHOnBOBA];
            }
            return [true, ""];
        }
        estimateBridgeTokenOutput(args) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.checkSwapSupported(args);
                }
                catch (e) {
                    return rejectPromise(e);
                }
                return this.calculateBridgeRate(args);
            });
        }
        buildBridgeTokenTransaction(args) {
            return __awaiter(this, void 0, void 0, function* () {
                const { addressTo } = args, tokenArgs = this.makeBridgeTokenArgs(args), { tokenFrom, tokenTo } = tokenArgs;
                if ((!addressTo) || addressTo === "") {
                    return rejectPromise(new Error("BridgeTransactionParams.addressTo cannot be empty string or undefined"));
                }
                args = Object.assign(Object.assign({}, args), { tokenFrom, tokenTo });
                let newTxn = this.chainId === ChainId.ETH
                    ? this.buildETHMainnetBridgeTxn(args, tokenArgs)
                    : this.buildL2BridgeTxn(args, tokenArgs);
                return newTxn
                    .then((txn) => GasUtils.populateGasParams(this.chainId, txn, "bridge"))
                    .catch(rejectPromise);
            });
        }
        executeBridgeTokenTransaction(args, signer) {
            return __awaiter(this, void 0, void 0, function* () {
                try {
                    yield this.checkSwapSupported(args);
                }
                catch (e) {
                    return rejectPromise(e);
                }
                const { tokenFrom, amountFrom, addressTo } = args, signerAddress = yield signer.getAddress();
                args.addressTo = addressTo !== null && addressTo !== void 0 ? addressTo : signerAddress;
                return this.checkCanBridge({
                    address: signerAddress,
                    token: tokenFrom,
                    amount: amountFrom,
                })
                    .then((canBridgeRes) => {
                    const [canBridge, err] = canBridgeRes;
                    if (!canBridge) {
                        return rejectPromise(err);
                    }
                    let txnProm = this.buildBridgeTokenTransaction(args);
                    return executePopulatedTransaction(txnProm, signer);
                })
                    .catch(rejectPromise);
            });
        }
        buildApproveTransaction(args) {
            return __awaiter(this, void 0, void 0, function* () {
                const [approveArgs, tokenAddress] = this.buildERC20ApproveArgs(args);
                return ERC20.buildApproveTransaction(approveArgs, { tokenAddress, chainId: this.chainId });
            });
        }
        executeApproveTransaction(args, signer) {
            return __awaiter(this, void 0, void 0, function* () {
                const [approveArgs, tokenAddress] = this.buildERC20ApproveArgs(args);
                return Promise.resolve(ERC20.approve(approveArgs, { tokenAddress, chainId: this.chainId }, signer)
                    .then((res) => res));
            });
        }
        getAllowanceForAddress(args) {
            return __awaiter(this, void 0, void 0, function* () {
                let { address, token } = args;
                let tokenAddress = token.address(this.chainId);
                return ERC20.allowanceOf(address, this.zapBridgeAddress, { tokenAddress, chainId: this.chainId });
            });
        }
        checkNeedsApprove(args) {
            return __awaiter(this, void 0, void 0, function* () {
                let { amount } = args;
                amount = amount !== null && amount !== void 0 ? amount : MAX_APPROVAL_AMOUNT.sub(1);
                const { address } = args;
                const [{ spender }, tokenAddress] = this.buildERC20ApproveArgs(args);
                return ERC20.allowanceOf(address, spender, { tokenAddress, chainId: this.chainId })
                    .then((allowance) => {
                    const res = [allowance.lt(amount), allowance];
                    return res;
                })
                    .catch(rejectPromise);
            });
        }
        checkHasBalance(args) {
            return __awaiter(this, void 0, void 0, function* () {
                const { address, amount } = args, [, tokenAddress] = this.buildERC20ApproveArgs(args);
                return ERC20.balanceOf(address, { tokenAddress, chainId: this.chainId })
                    .then((balance) => {
                    const res = [balance.gte(amount), balance];
                    return res;
                })
                    .catch(rejectPromise);
            });
        }
        checkCanBridge(args) {
            return __awaiter(this, void 0, void 0, function* () {
                const { token } = args;
                const hasBalanceRes = this.checkHasBalance(args)
                    .then((balanceRes) => {
                    const [hasBalance, balance] = balanceRes;
                    if (!hasBalance) {
                        let balanceEth = formatUnits(balance, token.decimals(this.chainId)).toString();
                        let ret = [false, new Error(`Balance of token ${token.symbol} is too low; current balance is ${balanceEth}`)];
                        return ret;
                    }
                    let ret = [true, null];
                    return ret;
                })
                    .catch(rejectPromise);
                return this.checkNeedsApprove(args)
                    .then((approveRes) => {
                    const [needsApprove, allowance] = approveRes;
                    if (needsApprove) {
                        let allowanceEth = formatUnits(allowance, token.decimals(this.chainId)).toString();
                        let ret = [false, new Error(`Spend allowance of Bridge too low for token ${token.symbol}; current allowance for Bridge is ${allowanceEth}`)];
                        return ret;
                    }
                    return hasBalanceRes;
                })
                    .catch(rejectPromise);
            });
        }
        buildERC20ApproveArgs(args) {
            const { token, amount } = args;
            let tokenAddr = (token instanceof BaseToken) || (token instanceof WrappedToken)
                ? token.address(this.chainId)
                : token;
            return [{
                    spender: this.zapBridgeAddress,
                    amount
                }, tokenAddr];
        }
        checkSwapSupported(args) {
            return __awaiter(this, void 0, void 0, function* () {
                const { chainIdTo, tokenFrom, tokenTo } = args, networkTo = Networks.fromChainId(chainIdTo);
                return new Promise((resolve, reject) => {
                    let [swapSupported, errReason] = this.swapSupported({ tokenFrom, chainIdTo, tokenTo });
                    if (!swapSupported) {
                        switch (errReason) {
                            case UnsupportedSwapReason.TokenNotSupported_From:
                                reject(`Network '${this.network.name}' does not support token ${tokenFrom.name} (param: tokenFrom)`);
                                break;
                            case UnsupportedSwapReason.TokenNotSupported_To:
                                reject(`Network '${networkTo.name}' (param: chainIdTo) does not support token ${tokenTo.name} (param: tokenTo)`);
                                break;
                            case UnsupportedSwapReason.NonmatchingSwapTypes:
                                reject(`param tokenFrom with swapType '${tokenFrom.swapType}' cannot be bridge to param tokenTo with swapType '${tokenTo.swapType}'`);
                                break;
                            default:
                                reject(errReason);
                                break;
                        }
                    }
                    resolve(true);
                });
            });
        }
        calculateBridgeRate(args) {
            return __awaiter(this, void 0, void 0, function* () {
                let { chainIdTo, amountFrom } = args;
                const toChainZapParams = { chainId: chainIdTo, signerOrProvider: newProviderForNetwork(chainIdTo) };
                const toChainZap = SynapseEntities.zapBridge(toChainZapParams);
                const { tokenFrom, tokenTo, tokenIndexFrom, tokenIndexTo, fromChainTokens } = this.makeBridgeTokenArgs(args);
                const mintBurnSwapTypes = [
                    SwapType.HIGH, SwapType.DOG, SwapType.JUMP,
                    SwapType.NFD, SwapType.OHM, SwapType.SOLAR,
                    SwapType.GMX,
                ];
                let [intermediateToken, bridgeConfigIntermediateToken] = (() => {
                    if (mintBurnSwapTypes.includes(tokenFrom.swapType)) {
                        return [tokenFrom, tokenFrom];
                    }
                    switch (tokenFrom.swapType) {
                        case SwapType.SYN:
                            return [Tokens.SYN, Tokens.SYN];
                        case SwapType.FRAX:
                            if (chainIdTo === ChainId.ETH) {
                                return [null, Tokens.FRAX];
                            }
                            else {
                                return [null, Tokens.SYN_FRAX];
                            }
                        case SwapType.ETH:
                            let intermediate;
                            if (chainIdTo === ChainId.ETH) {
                                intermediate = Tokens.WETH;
                            }
                            else {
                                intermediate = Tokens.NETH;
                            }
                            return [Tokens.NETH, intermediate];
                        case SwapType.AVAX:
                            return [Tokens.WAVAX, Tokens.WAVAX];
                        case SwapType.MOVR:
                            return [Tokens.WMOVR, Tokens.WMOVR];
                        default:
                            return [Tokens.NUSD, Tokens.NUSD];
                    }
                })();
                const bigNumTen = BigNumber.from(10);
                bridgeConfigIntermediateToken = bridgeConfigIntermediateToken !== null && bridgeConfigIntermediateToken !== void 0 ? bridgeConfigIntermediateToken : intermediateToken;
                const bridgeFeeRequest = this.bridgeConfigInstance.calculateSwapFee(bridgeConfigIntermediateToken.address(chainIdTo), chainIdTo, amountFrom.mul(bigNumTen.pow(18 - tokenFrom.decimals(this.chainId))));
                const checkEthy = (c, t) => BridgeUtils.isL2ETHChain(c) && t.swapType === SwapType.ETH;
                const ethToEth = this.chainId === ChainId.ETH && checkEthy(chainIdTo, tokenTo), ethFromEth = chainIdTo === ChainId.ETH && checkEthy(this.chainId, tokenFrom);
                let amountToReceive_from;
                switch (true) {
                    case amountFrom.eq(Zero):
                        amountToReceive_from = Zero;
                        break;
                    case ethToEth:
                    case Tokens.isMintBurnToken(tokenFrom):
                    case tokenFrom.isWrappedToken:
                        amountToReceive_from = amountFrom;
                        break;
                    case this.chainId === ChainId.ETH:
                        let liquidityAmounts = fromChainTokens.map((t) => tokenFrom.isEqual(t) ? amountFrom : Zero);
                        amountToReceive_from = yield this.zapBridgeInstance.calculateTokenAmount(liquidityAmounts, true);
                        break;
                    default:
                        amountToReceive_from = yield BridgeUtils.calculateSwapL2Zap(this.networkZapBridgeInstance, intermediateToken.address(this.chainId), tokenIndexFrom, 0, amountFrom);
                }
                let bridgeFee;
                try {
                    bridgeFee = yield bridgeFeeRequest;
                }
                catch (e) {
                    console.error(`Error in bridge fee request: ${e}`);
                    return null;
                }
                amountToReceive_from = BridgeUtils.subBigNumSafe(amountToReceive_from, bridgeFee);
                let amountToReceive_to;
                switch (true) {
                    case amountToReceive_from.isZero():
                        amountToReceive_to = Zero;
                        break;
                    case ethFromEth:
                    case Tokens.isMintBurnToken(tokenTo):
                    case tokenTo.isWrappedToken:
                        amountToReceive_to = amountToReceive_from;
                        break;
                    case chainIdTo === ChainId.ETH:
                        amountToReceive_to = yield toChainZap
                            .calculateRemoveLiquidityOneToken(amountToReceive_from, tokenIndexTo);
                        break;
                    default:
                        amountToReceive_to = yield BridgeUtils.calculateSwapL2Zap(toChainZap, intermediateToken.address(chainIdTo), 0, tokenIndexTo, amountToReceive_from);
                }
                let amountToReceive = amountToReceive_to;
                return { amountToReceive, bridgeFee };
            });
        }
        checkEasyArgs(args, zapBridge, easyDeposits, easyRedeems, easyDepositETH) {
            let castArgs = args;
            if (easyRedeems.includes(args.tokenTo.hash)) {
                return {
                    castArgs,
                    isEasy: true,
                    txn: zapBridge.populateTransaction.redeem(...BridgeUtils.makeEasyParams(castArgs, this.chainId, args.tokenTo))
                };
            }
            else if (easyDeposits.includes(args.tokenTo.hash)) {
                return {
                    castArgs,
                    isEasy: true,
                    txn: zapBridge.populateTransaction.deposit(...BridgeUtils.makeEasyParams(castArgs, this.chainId, args.tokenTo))
                };
            }
            else if (easyDepositETH.includes(args.tokenTo.hash)) {
                return {
                    castArgs,
                    isEasy: true,
                    txn: zapBridge.populateTransaction.depositETH(...BridgeUtils.depositETHParams(castArgs), { value: args.amountFrom })
                };
            }
            return { castArgs, isEasy: false };
        }
        buildETHMainnetBridgeTxn(args, tokenArgs) {
            const { addressTo, chainIdTo, amountFrom, amountTo } = args, zapBridge = SynapseEntities.l1BridgeZap({
                chainId: this.chainId,
                signerOrProvider: this.provider
            });
            let easyRedeems = [Tokens.SYN.hash], easyDeposits = [Tokens.HIGH.hash, Tokens.DOG.hash, Tokens.FRAX.hash], easyDepositETH = [Tokens.NETH.hash];
            if (args.tokenFrom.isEqual(Tokens.NUSD)) {
                easyDeposits.push(Tokens.NUSD.hash);
            }
            let { castArgs, isEasy, txn } = this.checkEasyArgs(args, zapBridge, easyDeposits, easyRedeems, easyDepositETH);
            if (isEasy && txn) {
                return txn;
            }
            const { transactionDeadline, bridgeTransactionDeadline, minToSwapDestFromOrigin, minToSwapDest, minToSwapOriginMediumSlippage, minToSwapDestFromOriginMediumSlippage, } = BridgeUtils.getSlippages(amountFrom, amountTo);
            switch (args.tokenTo.hash) {
                case Tokens.NUSD.hash:
                    if (!args.tokenFrom.isEqual(Tokens.NUSD)) {
                        const liquidityAmounts = tokenArgs.fromChainTokens.map((t) => {
                            return args.tokenFrom.isEqual(t) ? amountFrom : Zero;
                        });
                        return zapBridge.populateTransaction.zapAndDeposit(addressTo, chainIdTo, Tokens.NUSD.address(this.chainId), liquidityAmounts, minToSwapDest, transactionDeadline);
                    }
                    break;
                default:
                    if (BridgeUtils.isETHLikeToken(args.tokenTo) || args.tokenTo.isEqual(Tokens.WETH)) {
                        return zapBridge.populateTransaction.depositETHAndSwap(...BridgeUtils.depositETHParams(castArgs), 0, tokenArgs.tokenIndexTo, minToSwapDestFromOrigin, bridgeTransactionDeadline, { value: amountFrom });
                    }
                    const liquidityAmounts = tokenArgs.fromChainTokens.map((t) => {
                        return args.tokenFrom.isEqual(t) ? amountFrom : Zero;
                    });
                    return zapBridge.populateTransaction.zapAndDepositAndSwap(addressTo, chainIdTo, Tokens.NUSD.address(this.chainId), liquidityAmounts, minToSwapOriginMediumSlippage, transactionDeadline, 0, tokenArgs.tokenIndexTo, minToSwapDestFromOriginMediumSlippage, bridgeTransactionDeadline);
            }
        }
        buildL2BridgeTxn(args, tokenArgs) {
            const { chainIdTo, amountFrom, amountTo } = args, zapBridge = SynapseEntities.l2BridgeZap({
                chainId: this.chainId,
                signerOrProvider: this.provider
            });
            if (tokenArgs.tokenFrom.isEqual(Tokens.AVWETH)) {
                tokenArgs.tokenFrom = Tokens.WETH_E;
            }
            let easyDeposits = [], easyRedeems = [Tokens.SYN.hash, Tokens.HIGH.hash, Tokens.DOG.hash, Tokens.FRAX.hash], easyDepositETH = [];
            if (args.tokenFrom.isEqual(Tokens.NUSD)) {
                easyRedeems.push(Tokens.NUSD.hash);
            }
            BridgeUtils.DepositIfChainTokens.forEach((args) => {
                let { chainId, tokens, depositEth, altChainId } = args;
                let hasAltChain = typeof altChainId !== 'undefined', tokenHashes = tokens.map((t) => t.hash);
                if (this.chainId === chainId) {
                    depositEth
                        ? easyDepositETH.push(...tokenHashes)
                        : easyDeposits.push(...tokenHashes);
                }
                else {
                    if (hasAltChain) {
                        if (this.chainId === altChainId)
                            easyRedeems.push(...tokenHashes);
                    }
                    else {
                        easyRedeems.push(...tokenHashes);
                    }
                }
            });
            let { castArgs, isEasy, txn } = this.checkEasyArgs(args, zapBridge, easyDeposits, easyRedeems, easyDepositETH);
            if (isEasy && txn) {
                return txn;
            }
            const { transactionDeadline, bridgeTransactionDeadline, minToSwapOriginHighSlippage, minToSwapDestFromOriginHighSlippage, minToSwapDest, } = BridgeUtils.getSlippages(amountFrom, amountTo);
            const easyRedeemAndSwap = (baseToken) => zapBridge.populateTransaction.redeemAndSwap(...BridgeUtils.makeEasyParams(castArgs, this.chainId, baseToken), 0, tokenArgs.tokenIndexTo, minToSwapDest, transactionDeadline);
            const easySwapAndRedeemAndSwap = (baseToken, withValueOverride) => zapBridge.populateTransaction.swapAndRedeemAndSwap(...BridgeUtils.makeEasySubParams(castArgs, this.chainId, baseToken), tokenArgs.tokenIndexFrom, 0, amountFrom, minToSwapOriginHighSlippage, transactionDeadline, 0, tokenArgs.tokenIndexTo, minToSwapDestFromOriginHighSlippage, bridgeTransactionDeadline, BridgeUtils.makeOverrides(amountFrom, withValueOverride));
            switch (args.tokenTo.hash) {
                case Tokens.NUSD.hash:
                    return zapBridge.populateTransaction.swapAndRedeem(...BridgeUtils.makeEasySubParams(castArgs, this.chainId, Tokens.NUSD), tokenArgs.tokenIndexFrom, 0, amountFrom, minToSwapOriginHighSlippage, transactionDeadline);
                case Tokens.GMX.hash:
                    let params = BridgeUtils.makeEasyParams(castArgs, this.chainId, Tokens.GMX);
                    switch (this.chainId) {
                        case ChainId.ARBITRUM:
                            return zapBridge.populateTransaction.deposit(...params);
                        default:
                            let [addrTo, chainTo, , amount] = params;
                            return this.bridgeInstance.populateTransaction.redeem(addrTo, chainTo, Tokens.GMX.wrapperAddress(this.chainId), amount);
                    }
                default:
                    if (chainIdTo === ChainId.ETH) {
                        if ((BridgeUtils.isL2ETHChain(this.chainId)) && (args.tokenFrom.swapType === SwapType.ETH)) {
                            if (args.tokenFrom.isEqual(Tokens.NETH)) {
                                return zapBridge.populateTransaction.redeem(...BridgeUtils.makeEasyParams(castArgs, this.chainId, Tokens.NETH));
                            }
                            else if (BridgeUtils.isETHLikeToken(args.tokenFrom)) {
                                return zapBridge.populateTransaction.swapAndRedeem(...BridgeUtils.makeEasySubParams(castArgs, this.chainId, Tokens.NETH), tokenArgs.tokenIndexFrom, 0, amountFrom, minToSwapOriginHighSlippage, transactionDeadline);
                            }
                            else {
                                return zapBridge.populateTransaction.swapETHAndRedeem(...BridgeUtils.makeEasySubParams(castArgs, this.chainId, Tokens.NETH), tokenArgs.tokenIndexFrom, 0, amountFrom, minToSwapOriginHighSlippage, transactionDeadline, { value: amountFrom });
                            }
                        }
                        else if (args.tokenFrom.isEqual(Tokens.NUSD)) {
                            return zapBridge.populateTransaction.redeemAndRemove(...BridgeUtils.makeEasySubParams(castArgs, this.chainId, Tokens.NUSD), amountFrom, tokenArgs.tokenIndexTo, minToSwapDest, transactionDeadline);
                        }
                        else {
                            return zapBridge.populateTransaction.swapAndRedeemAndRemove(...BridgeUtils.makeEasySubParams(castArgs, this.chainId, Tokens.NUSD), tokenArgs.tokenIndexFrom, 0, amountFrom, minToSwapOriginHighSlippage, transactionDeadline, tokenArgs.tokenIndexTo, minToSwapDestFromOriginHighSlippage, bridgeTransactionDeadline);
                        }
                    }
                    else {
                        if (args.tokenFrom.isEqual(Tokens.NUSD)) {
                            return easyRedeemAndSwap(Tokens.NUSD);
                        }
                        else if (args.tokenFrom.isEqual(Tokens.NETH)) {
                            return easyRedeemAndSwap(Tokens.NETH);
                        }
                        else if (args.tokenFrom.swapType === SwapType.ETH) {
                            if (BridgeUtils.isETHLikeToken(args.tokenFrom)) {
                                return easySwapAndRedeemAndSwap(Tokens.NETH, false);
                            }
                            else {
                                return zapBridge.populateTransaction.swapETHAndRedeemAndSwap(...BridgeUtils.makeEasySubParams(castArgs, this.chainId, Tokens.NETH), tokenArgs.tokenIndexFrom, 0, amountFrom, minToSwapOriginHighSlippage, transactionDeadline, 0, tokenArgs.tokenIndexTo, minToSwapDestFromOriginHighSlippage, bridgeTransactionDeadline, { value: amountFrom });
                            }
                        }
                        else {
                            return easySwapAndRedeemAndSwap(Tokens.NUSD, false);
                        }
                    }
            }
        }
        makeBridgeTokenArgs(args) {
            let { tokenFrom, tokenTo, chainIdTo } = args;
            const swapparoo = (t, check, swappy) => t.isEqual(check) ? swappy : t, swappadoo = (check, swappy) => (t1, t2) => [swapparoo(t1, check, swappy), swapparoo(t2, check, swappy)];
            let kangaroo;
            switch (tokenFrom.swapType) {
                case SwapType.ETH:
                    kangaroo = swappadoo(Tokens.ETH, Tokens.WETH);
                    break;
                case SwapType.AVAX:
                    kangaroo = swappadoo(Tokens.AVAX, Tokens.WAVAX);
                    break;
                case SwapType.MOVR:
                    kangaroo = swappadoo(Tokens.MOVR, Tokens.WMOVR);
                    break;
                default:
                    kangaroo = (t1, t2) => [t1, t2];
            }
            [tokenFrom, tokenTo] = kangaroo(tokenFrom, tokenTo);
            const findSymbol = (tokA, tokB) => {
                let compareTok = tokB;
                if (tokB.isEqual(Tokens.WETH_E)) {
                    compareTok = Tokens.AVWETH;
                }
                else if (tokB.isEqual(Tokens.ETH)) {
                    compareTok = Tokens.WETH;
                }
                else if (tokB.isWrappedToken) {
                    compareTok = tokB.underlyingToken;
                }
                return tokA.isEqual(compareTok);
            };
            const makeTokenArgs = (chainId, t) => {
                var _a, _b;
                let toks = (_b = (_a = SwapPools.bridgeSwappableTypePoolsByChain[chainId]) === null || _a === void 0 ? void 0 : _a[t.swapType]) === null || _b === void 0 ? void 0 : _b.poolTokens, idx = toks.findIndex((tok) => findSymbol(tok, t));
                return [toks, idx];
            };
            const [fromChainTokens, tokenIndexFrom] = makeTokenArgs(this.chainId, tokenFrom), [toChainTokens, tokenIndexTo] = makeTokenArgs(chainIdTo, tokenTo);
            return {
                fromChainTokens,
                toChainTokens,
                tokenFrom,
                tokenTo,
                tokenIndexFrom,
                tokenIndexTo
            };
        }
    }
    Bridge.SynapseBridge = SynapseBridge;
    const REQUIRED_CONFS = {
        [ChainId.ETH]: 7,
        [ChainId.OPTIMISM]: 1,
        [ChainId.BSC]: 14,
        [ChainId.POLYGON]: 128,
        [ChainId.FANTOM]: 5,
        [ChainId.BOBA]: 1,
        [ChainId.MOONBEAM]: 21,
        [ChainId.MOONRIVER]: 21,
        [ChainId.ARBITRUM]: 40,
        [ChainId.AVALANCHE]: 5,
        [ChainId.HARMONY]: 1,
    };
    function getRequiredConfirmationsForBridge(network) {
        var _a;
        let chainId = network instanceof Networks.Network ? network.chainId : network;
        return (_a = REQUIRED_CONFS[chainId]) !== null && _a !== void 0 ? _a : -1;
    }
    Bridge.getRequiredConfirmationsForBridge = getRequiredConfirmationsForBridge;
})(Bridge || (Bridge = {}));
